@@ -338,7 +338,7 @@ PluginConfig = Dict[str, Union[str, int, bool]]
 
 
 class JsonBaseSchema(TypedDict):
-    type: str
+    type: Any
     key: Optional[str]
     title: Optional[str]
     description: Optional[str]
@@ -357,30 +357,30 @@ class PluginJsonBaseSchema(JsonBaseSchema):
 
 
 class PluginJsonSchemaString(PluginJsonBaseSchema):
-    type: Literal["string"]
+    type: str
     format: Optional[str]
     minLength: Optional[int]
     maxLength: Optional[int]
 
 
 class PluginJsonSchemaNumber(PluginJsonBaseSchema):
-    type: Literal["number"]
+    type: int
     minimum: Optional[int]
     maximum: Optional[int]
 
 
 class PluginJsonSchemaBoolean(PluginJsonBaseSchema):
-    type: Literal["boolean"]
+    type: bool
 
 
 class PluginJsonSchemaEnum(PluginJsonBaseSchema):
-    type: Literal["string"]
+    type: str
     enum: List[str]
     multiple: Optional[bool]
 
 
 class PluginJsonSchemaObject(PluginJsonBaseSchema):
-    type: Literal["object"]
+    type: Dict[str, Any]
     opened: Optional[bool]
     properties: Optional[Dict[str, "PluginJsonSchema"]]
 
@@ -394,9 +394,7 @@ PluginJsonSchema = Union[
 ]
 
 
-class PluginJsonSchemaForm(TypedDict, total=False):
-    pass
-
+PluginJsonSchemaForm = Dict[str, PluginJsonSchema]
 
 class PluginRootSchema(TypedDict):
     schema: PluginJsonSchemaForm
@@ -404,30 +402,30 @@ class PluginRootSchema(TypedDict):
 
 # json schema types
 class JsonSchemaString(JsonBaseSchema):
-    type: Literal["string"]
+    type: str
     format: Optional[str]
     minLength: Optional[int]
     maxLength: Optional[int]
 
 
 class JsonSchemaNumber(JsonBaseSchema):
-    type: Literal["number"]
+    type: int
     minimum: Optional[int]
     maximum: Optional[int]
 
 
 class JsonSchemaBoolean(JsonBaseSchema):
-    type: Literal["boolean"]
+    type: bool
 
 
 class JsonSchemaEnum(JsonBaseSchema):
-    type: Literal["string"]
+    type: str
     enum: List[str]
     multiple: Optional[bool]
 
 
 class JsonSchemaObject(JsonBaseSchema):
-    type: Literal["object"]
+    type: Dict[str, Any]
     opened: Optional[bool]
     properties: Optional[Dict[str, "JsonSchema"]]
 
@@ -446,7 +444,7 @@ class JsonSchemaAnyOf(TypedDict):
 
 
 class JsonSchemaArray(JsonBaseSchema):
-    type: Literal["array"]
+    type: List[Any]
     opened: Optional[bool]
     items: Optional[Union["JsonSchema", JsonSchemaAnyOf]]
 
@@ -558,12 +556,12 @@ class StorageController(Protocol):
 # logger
 @runtime_checkable
 class PluginLogger(Protocol):
-    def log(self, *args) -> None: ...
-    def error(self, *args) -> None: ...
-    def warn(self, *args) -> None: ...
-    def attention(self, *args) -> None: ...
-    def debug(self, *args) -> None: ...
-    def trace(self, *args) -> None: ...
+    def log(self, *args: Any) -> None: ...
+    def error(self, *args: Any) -> None: ...
+    def warn(self, *args: Any) -> None: ...
+    def attention(self, *args: Any) -> None: ...
+    def debug(self, *args: Any) -> None: ...
+    def trace(self, *args: Any) -> None: ...
 
 
 # config service
@@ -612,17 +610,17 @@ class PluginConfigService(Protocol):
 # manager todo: improve typing
 @runtime_checkable
 class SystemManager(Protocol):
-    def on(self, event: str, listener: Callable) -> "SystemManager": ...
-    def once(self, event: str, listener: Callable) -> "SystemManager": ...
-    def remove_listener(self, event: str, listener: Callable) -> "SystemManager": ...
+    def on(self, event: str, listener: Callback) -> "SystemManager": ...
+    def once(self, event: str, listener: Callback) -> "SystemManager": ...
+    def remove_listener(self, event: str, listener: Callback) -> "SystemManager": ...
     def remove_all_listeners(self, event: str) -> "SystemManager": ...
 
 
 @runtime_checkable
 class PluginsManager(Protocol):  # todo: add methods
-    def on(self, event: str, listener: Callable) -> "PluginsManager": ...
-    def once(self, event: str, listener: Callable) -> "PluginsManager": ...
-    def remove_listener(self, event: str, listener: Callable) -> "PluginsManager": ...
+    def on(self, event: str, listener: Callback) -> "PluginsManager": ...
+    def once(self, event: str, listener: Callback) -> "PluginsManager": ...
+    def remove_listener(self, event: str, listener: Callback) -> "PluginsManager": ...
     def remove_all_listeners(self, event: str) -> "PluginsManager": ...
 
 
@@ -649,14 +647,14 @@ class CameraDevice(Protocol):
     has_object_detector: bool
     has_ptz: bool
 
-    on_connected: Observable
-    on_light_switched: Observable
-    on_motion_detected: Observable
-    on_audio_detected: Observable
-    on_object_detected: Observable
-    on_doorbell_pressed: Observable
-    on_siren_detected: Observable
-    on_battery_changed: Observable
+    on_connected: Observable[bool]
+    on_light_switched: Observable[LightState]
+    on_motion_detected: Observable[MotionState]
+    on_audio_detected: Observable[AudioState]
+    on_object_detected: Observable[ObjectState]
+    on_doorbell_pressed: Observable[DoorbellState]
+    on_siren_detected: Observable[SirenState]
+    on_battery_changed: Observable[BatteryState]
 
     stream_source: CameraSource
     snapshot_source: CameraSource
@@ -738,8 +736,8 @@ class CameraDevice(Protocol):
         event_data: BatterySetEvent,
     ) -> None: ...
     async def update_state(self, state_name: str, event_data: Any) -> None: ...
-    def on_state_change(self, state_name: StateNames) -> Observable: ...
-    def on_property_change(self, property_name: str) -> Observable: ...
+    def on_state_change(self, state_name: StateNames) -> Observable[Any]: ...
+    def on_property_change(self, property_name: str) -> Observable[Any]: ...
     def remove_all_listeners(self) -> None: ...
 
 
@@ -823,10 +821,10 @@ class PluginAPI(Protocol):
     system_manager: SystemManager
     plugins_manager: PluginsManager
 
-    def on(self, event: APIEventType, listener: Callable) -> "PluginAPI": ...
-    def once(self, event: APIEventType, listener: Callable) -> "PluginAPI": ...
+    def on(self, event: APIEventType, listener: Callback) -> "PluginAPI": ...
+    def once(self, event: APIEventType, listener: Callback) -> "PluginAPI": ...
     def remove_listener(
-        self, event: APIEventType, listener: Callable
+        self, event: APIEventType, listener: Callback
     ) -> "PluginAPI": ...
     def remove_all_listeners(
         self, event: Optional[APIEventType] = None
@@ -836,7 +834,7 @@ class PluginAPI(Protocol):
 # base plugin
 class BasePlugin(ABC):
     @abstractmethod
-    async def on_form_submit(self, action_id: str, payload: Any) -> Optional[Dict]:
+    async def on_form_submit(self, action_id: str, payload: Any) -> Optional[Dict[Any, Any]]:
         pass
 
     @abstractmethod
